@@ -30,6 +30,7 @@
 #include "agg_image.h"
 #include "army_troop.h"
 #include "audio_manager.h"
+#include "castle.h"
 #include "castle_building_info.h"
 #include "cursor.h"
 #include "dialog.h"
@@ -76,16 +77,32 @@ namespace
             return {};
         }
     }
+
+    const char * GetBuildConditionDescription( const BuildingStatus status )
+    {
+        switch ( status ) {
+        case BuildingStatus::NOT_TODAY:
+            return _( "Cannot build. You have already built here today." );
+
+        case BuildingStatus::NEED_CASTLE:
+            return _( "For this action it is necessary to build a castle first." );
+
+        default:
+            break;
+        }
+
+        return nullptr;
+    }
 }
 
-struct buildstats_t
+struct BuildingStats
 {
     uint32_t id2;
     uint8_t race;
-    cost_t cost;
+    Cost cost;
 };
 
-const buildstats_t _builds[] = {
+const BuildingStats buildingStats[] = {
     // id                             gold wood mercury ore sulfur crystal gems
     { BUILD_THIEVESGUILD, Race::ALL, { 750, 5, 0, 0, 0, 0, 0 } },
     { BUILD_TAVERN, Race::ALL, { 500, 5, 0, 0, 0, 0, 0 } },
@@ -190,7 +207,7 @@ const buildstats_t _builds[] = {
 Funds BuildingInfo::GetCost( uint32_t build, int race )
 {
     Funds payment;
-    const buildstats_t * ptr = &_builds[0];
+    const BuildingStats * ptr = &buildingStats[0];
 
     while ( BUILD_NOTHING != ptr->id2 && !( ptr->id2 == build && ( !race || ( race & ptr->race ) ) ) )
         ++ptr;
@@ -208,34 +225,34 @@ Funds BuildingInfo::GetCost( uint32_t build, int race )
     return payment;
 }
 
-BuildingInfo::BuildingInfo( const Castle & c, const building_t b )
+BuildingInfo::BuildingInfo( const Castle & c, const BuildingType b )
     : castle( c )
-    , building( b )
+    , _buildingType( b )
     , area( 0, 0, 135, 70 )
-    , bcond( ALLOW_BUILD )
+    , _status( BuildingStatus::ALLOW_BUILD )
 {
-    if ( isDwelling( building ) )
-        building = castle.GetActualDwelling( b );
+    if ( isDwelling( _buildingType ) )
+        _buildingType = castle.GetActualDwelling( b );
 
-    building = castle.isBuild( b ) ? castle.GetUpgradeBuilding( b ) : b;
+    _buildingType = castle.isBuild( b ) ? castle.GetUpgradeBuilding( b ) : b;
 
-    if ( BUILD_TAVERN == building && Race::NECR == castle.GetRace() ) {
+    if ( BUILD_TAVERN == _buildingType && Race::NECR == castle.GetRace() ) {
         const GameVersion version = Settings::Get().getCurrentMapInfo().version;
-        building = ( version == GameVersion::PRICE_OF_LOYALTY || version == GameVersion::RESURRECTION ) ? BUILD_SHRINE : BUILD_NOTHING;
+        _buildingType = ( version == GameVersion::PRICE_OF_LOYALTY || version == GameVersion::RESURRECTION ) ? BUILD_SHRINE : BUILD_NOTHING;
     }
 
-    bcond = castle.CheckBuyBuilding( building );
+    _status = castle.CheckBuyBuilding( _buildingType );
 
     // generate description
-    if ( BUILD_DISABLE == bcond )
+    if ( _status == BuildingStatus::BUILD_DISABLE || _status == BuildingStatus::SHIPYARD_NOT_ALLOWED )
         description = GetConditionDescription();
     else {
-        description = getBuildingDescription( castle.GetRace(), building );
+        description = getBuildingDescription( castle.GetRace(), _buildingType );
     }
 
     // fix area for captain
     if ( b == BUILD_CAPTAIN ) {
-        const fheroes2::Sprite & sprite = fheroes2::AGG::GetICN( ICN::Get4Captain( castle.GetRace() ), ( building & BUILD_CAPTAIN ? 1 : 0 ) );
+        const fheroes2::Sprite & sprite = fheroes2::AGG::GetICN( ICN::Get4Captain( castle.GetRace() ), ( _buildingType & BUILD_CAPTAIN ? 1 : 0 ) );
         area.width = sprite.width();
         area.height = sprite.height();
     }
@@ -284,7 +301,7 @@ std::string BuildingInfo::getBuildingDescription( const int race, const uint32_t
         }
     }
     else {
-        description = fheroes2::getBuildingDescription( race, static_cast<building_t>( buildingId ) );
+        description = fheroes2::getBuildingDescription( race, static_cast<BuildingType>( buildingId ) );
 
         switch ( buildingId ) {
         case BUILD_WELL:
@@ -311,7 +328,7 @@ std::string BuildingInfo::getBuildingDescription( const int race, const uint32_t
 void BuildingInfo::RedrawCaptain() const
 {
     fheroes2::Display & display = fheroes2::Display::instance();
-    if ( bcond == ALREADY_BUILT ) {
+    if ( _status == BuildingStatus::ALREADY_BUILT ) {
         const fheroes2::Sprite & captainSprite = fheroes2::AGG::GetICN( ICN::Get4Captain( castle.GetRace() ), 1 );
         const fheroes2::Sprite & flag = fheroes2::AGG::GetICN( ICN::GetFlagIcnId( castle.GetColor() ), 0 );
 
@@ -324,12 +341,12 @@ void BuildingInfo::RedrawCaptain() const
     }
 
     // indicator
-    if ( bcond == ALREADY_BUILT ) {
+    if ( _status == BuildingStatus::ALREADY_BUILT ) {
         const fheroes2::Sprite & spriteAllow = fheroes2::AGG::GetICN( ICN::TOWNWIND, 11 );
         fheroes2::Blit( spriteAllow, display, area.x + 83 - 4 - spriteAllow.width(), area.y + 79 - 2 - spriteAllow.height() );
     }
-    else if ( bcond != ALLOW_BUILD ) {
-        if ( LACK_RESOURCES == bcond ) {
+    else if ( _status != BuildingStatus::ALLOW_BUILD ) {
+        if ( BuildingStatus::LACK_RESOURCES == _status ) {
             const fheroes2::Sprite & spriteMoney = fheroes2::AGG::GetICN( ICN::TOWNWIND, 13 );
             fheroes2::Blit( spriteMoney, display, area.x + 83 - 4 + 1 - spriteMoney.width(), area.y + 79 - 3 - spriteMoney.height() );
         }
@@ -342,17 +359,17 @@ void BuildingInfo::RedrawCaptain() const
 
 void BuildingInfo::Redraw() const
 {
-    if ( BUILD_CAPTAIN == building ) {
+    if ( BUILD_CAPTAIN == _buildingType ) {
         RedrawCaptain();
         return;
     }
 
     fheroes2::Display & display = fheroes2::Display::instance();
-    const int index = fheroes2::getIndexBuildingSprite( static_cast<building_t>( building ) );
+    const int index = fheroes2::getIndexBuildingSprite( static_cast<BuildingType>( _buildingType ) );
 
     const fheroes2::Sprite & buildingFrame = fheroes2::AGG::GetICN( ICN::BLDGXTRA, 0 );
     fheroes2::Blit( buildingFrame, display, area.x, area.y );
-    if ( BUILD_DISABLE == bcond ) {
+    if ( _status == BuildingStatus::BUILD_DISABLE || _status == BuildingStatus::SHIPYARD_NOT_ALLOWED ) {
         const fheroes2::Point offset( 6, 59 );
         fheroes2::Sprite grayedOut = fheroes2::Crop( buildingFrame, offset.x, offset.y, 125, 12 );
         fheroes2::ApplyPalette( grayedOut, PAL::GetPalette( PAL::PaletteType::GRAY ) );
@@ -361,7 +378,7 @@ void BuildingInfo::Redraw() const
     }
 
     // build image
-    if ( BUILD_NOTHING == building ) {
+    if ( BUILD_NOTHING == _buildingType ) {
         const bool isEvilInterface = Settings::Get().isEvilInterfaceEnabled();
         const fheroes2::Sprite & buildBackground = fheroes2::AGG::GetICN( isEvilInterface ? ICN::CASLXTRA_EVIL : ICN::CASLXTRA, 0 );
         fheroes2::Copy( buildBackground, 0, 0, display, area.x, area.y, buildBackground.width(), buildBackground.height() );
@@ -371,19 +388,19 @@ void BuildingInfo::Redraw() const
     fheroes2::Blit( fheroes2::AGG::GetICN( ICN::Get4Building( castle.GetRace() ), index ), display, area.x + 1, area.y + 1 );
 
     // indicator
-    if ( bcond == ALREADY_BUILT ) {
+    if ( _status == BuildingStatus::ALREADY_BUILT ) {
         const fheroes2::Sprite & spriteAllow = fheroes2::AGG::GetICN( ICN::TOWNWIND, 11 );
         fheroes2::Blit( spriteAllow, display, area.x + buildingFrame.width() - 5 - spriteAllow.width(), area.y + 58 - 2 - spriteAllow.height() );
     }
-    else if ( bcond == BUILD_DISABLE ) {
+    else if ( _status == BuildingStatus::BUILD_DISABLE || _status == BuildingStatus::SHIPYARD_NOT_ALLOWED ) {
         const fheroes2::Sprite & spriteDeny = fheroes2::AGG::GetICN( ICN::TOWNWIND, 12 );
         fheroes2::Sprite disabledSprite( spriteDeny );
         fheroes2::ApplyPalette( disabledSprite, PAL::GetPalette( PAL::PaletteType::GRAY ) );
         fheroes2::ApplyPalette( disabledSprite, PAL::GetPalette( PAL::PaletteType::DARKENING ) );
         fheroes2::Blit( disabledSprite, display, area.x + buildingFrame.width() - 5 + 1 - spriteDeny.width(), area.y + 58 - 2 - spriteDeny.height() );
     }
-    else if ( bcond != ALLOW_BUILD ) {
-        if ( LACK_RESOURCES == bcond ) {
+    else if ( _status != BuildingStatus::ALLOW_BUILD ) {
+        if ( BuildingStatus::LACK_RESOURCES == _status ) {
             const fheroes2::Sprite & spriteMoney = fheroes2::AGG::GetICN( ICN::TOWNWIND, 13 );
             fheroes2::Blit( spriteMoney, display, area.x + buildingFrame.width() - 5 + 1 - spriteMoney.width(), area.y + 58 - 3 - spriteMoney.height() );
         }
@@ -400,13 +417,13 @@ void BuildingInfo::Redraw() const
         fheroes2::Copy( textBackground, 0, 0, display, area.x, area.y + 58, textBackground.width(), textBackground.height() );
     }
 
-    const fheroes2::Text buildingName( Castle::GetStringBuilding( building, castle.GetRace() ), fheroes2::FontType::smallWhite() );
+    const fheroes2::Text buildingName( Castle::GetStringBuilding( _buildingType, castle.GetRace() ), fheroes2::FontType::smallWhite() );
     buildingName.draw( area.x + 68 - buildingName.width() / 2, area.y + 61, display );
 }
 
 const char * BuildingInfo::GetName() const
 {
-    return Castle::GetStringBuilding( building, castle.GetRace() );
+    return Castle::GetStringBuilding( _buildingType, castle.GetRace() );
 }
 
 bool BuildingInfo::QueueEventProcessing( fheroes2::ButtonBase & exitButton ) const
@@ -414,12 +431,12 @@ bool BuildingInfo::QueueEventProcessing( fheroes2::ButtonBase & exitButton ) con
     LocalEvent & le = LocalEvent::Get();
 
     if ( le.MouseClickLeft( area ) ) {
-        if ( bcond == LACK_RESOURCES || bcond == ALLOW_BUILD ) {
+        if ( _status == BuildingStatus::LACK_RESOURCES || _status == BuildingStatus::ALLOW_BUILD ) {
             const fheroes2::ButtonRestorer exitRestorer( exitButton );
             return DialogBuyBuilding( true );
         }
     }
-    else if ( le.MousePressRight( area ) ) {
+    else if ( le.isMouseRightButtonPressedInArea( area ) ) {
         DialogBuyBuilding( false );
     }
     return false;
@@ -427,7 +444,7 @@ bool BuildingInfo::QueueEventProcessing( fheroes2::ButtonBase & exitButton ) con
 
 bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
 {
-    if ( building == BUILD_NOTHING ) {
+    if ( _buildingType == BUILD_NOTHING ) {
         return false;
     }
 
@@ -435,7 +452,7 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
 
     std::string extendedDescription = description;
 
-    if ( ALLOW_BUILD != bcond ) {
+    if ( BuildingStatus::ALLOW_BUILD != _status ) {
         const std::string & ext = GetConditionDescription();
         if ( !ext.empty() && ext != description ) {
             extendedDescription.append( "\n\n" );
@@ -446,7 +463,11 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
     const fheroes2::Text descriptionText( std::move( extendedDescription ), fheroes2::FontType::normalWhite() );
 
     // prepare requirement build string
-    std::string requirement = fheroes2::getBuildingRequirementString( castle.GetRace(), static_cast<building_t>( building ) );
+    std::string requirement;
+
+    if ( _status != BuildingStatus::BUILD_DISABLE ) {
+        requirement = fheroes2::getBuildingRequirementString( castle.GetRace(), static_cast<BuildingType>( _buildingType ) );
+    }
 
     const bool requirementsPresent = !requirement.empty();
 
@@ -460,7 +481,7 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
         requirementHeight = requirementTitle.height() + requirementText.height( BOXAREA_WIDTH ) + elementOffset;
     }
 
-    Resource::BoxSprite rbs( PaymentConditions::BuyBuilding( castle.GetRace(), building ), BOXAREA_WIDTH );
+    Resource::BoxSprite rbs( PaymentConditions::BuyBuilding( castle.GetRace(), _buildingType ), BOXAREA_WIDTH );
 
     const fheroes2::Sprite & buildingFrame = fheroes2::AGG::GetICN( ICN::BLDGXTRA, 0 );
 
@@ -489,7 +510,7 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
     fheroes2::Blit( buildingFrame, display, pos.x, pos.y );
 
     const fheroes2::Sprite & buildingImage
-        = fheroes2::AGG::GetICN( ICN::Get4Building( castle.GetRace() ), fheroes2::getIndexBuildingSprite( static_cast<building_t>( building ) ) );
+        = fheroes2::AGG::GetICN( ICN::Get4Building( castle.GetRace() ), fheroes2::getIndexBuildingSprite( static_cast<BuildingType>( _buildingType ) ) );
     pos.x = dialogRoi.x + ( dialogRoi.width - buildingImage.width() ) / 2;
     pos.y += 1;
     fheroes2::Blit( buildingImage, display, pos.x, pos.y );
@@ -519,7 +540,7 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
     rbs.Redraw();
 
     if ( buttons ) {
-        if ( ALLOW_BUILD != castle.CheckBuyBuilding( building ) ) {
+        if ( BuildingStatus::ALLOW_BUILD != castle.CheckBuyBuilding( _buildingType ) ) {
             buttonOkay.disable();
         }
 
@@ -538,12 +559,12 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
 
     LocalEvent & le = LocalEvent::Get();
     while ( le.HandleEvents() ) {
-        if ( !buttons && !le.MousePressRight() ) {
+        if ( !buttons && !le.isMouseRightButtonPressed() ) {
             break;
         }
 
-        le.MousePressLeft( buttonOkay.area() ) ? buttonOkay.drawOnPress() : buttonOkay.drawOnRelease();
-        le.MousePressLeft( buttonCancel.area() ) ? buttonCancel.drawOnPress() : buttonCancel.drawOnRelease();
+        le.isMouseLeftButtonPressedInArea( buttonOkay.area() ) ? buttonOkay.drawOnPress() : buttonOkay.drawOnRelease();
+        le.isMouseLeftButtonPressedInArea( buttonCancel.area() ) ? buttonCancel.drawOnPress() : buttonCancel.drawOnRelease();
 
         if ( buttonOkay.isEnabled() && ( Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) || le.MouseClickLeft( buttonOkay.area() ) ) ) {
             return true;
@@ -553,10 +574,10 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
             break;
         }
 
-        if ( buttonOkay.isVisible() && le.MousePressRight( buttonOkay.area() ) ) {
+        if ( buttonOkay.isVisible() && le.isMouseRightButtonPressedInArea( buttonOkay.area() ) ) {
             fheroes2::showStandardTextMessage( _( "Okay" ), GetConditionDescription(), Dialog::ZERO );
         }
-        else if ( buttonCancel.isVisible() && le.MousePressRight( buttonCancel.area() ) ) {
+        else if ( buttonCancel.isVisible() && le.isMouseRightButtonPressedInArea( buttonCancel.area() ) ) {
             fheroes2::showStandardTextMessage( _( "Cancel" ), _( "Exit this menu without doing anything." ), Dialog::ZERO );
         }
     }
@@ -564,84 +585,69 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
     return false;
 }
 
-const char * GetBuildConditionDescription( int bcond )
-{
-    switch ( bcond ) {
-    case NOT_TODAY:
-        return _( "Cannot build. You have already built here today." );
-
-    case NEED_CASTLE:
-        return _( "For this action it is necessary to build a castle first." );
-
-    default:
-        break;
-    }
-
-    return nullptr;
-}
-
 std::string BuildingInfo::GetConditionDescription() const
 {
-    std::string res;
+    switch ( _status ) {
+    case BuildingStatus::NOT_TODAY:
+    case BuildingStatus::NEED_CASTLE:
+        return GetBuildConditionDescription( _status );
 
-    switch ( bcond ) {
-    case NOT_TODAY:
-    case NEED_CASTLE:
-        return GetBuildConditionDescription( bcond );
+    case BuildingStatus::SHIPYARD_NOT_ALLOWED: {
+        assert( _buildingType == BUILD_SHIPYARD );
+        std::string res = _( "Cannot build %{name}. The castle is too far away from an ocean." );
+        StringReplace( res, "%{name}", Castle::GetStringBuilding( BUILD_SHIPYARD, castle.GetRace() ) );
+        return res;
+    }
 
-    case BUILD_DISABLE:
-        if ( building == BUILD_SHIPYARD ) {
-            res = _( "Cannot build %{name}. The castle is too far away from an ocean." );
-            StringReplace( res, "%{name}", Castle::GetStringBuilding( BUILD_SHIPYARD, castle.GetRace() ) );
-        }
-        else {
-            // TODO: Add future disabled buildings here when it is made possible in the Editor.
-            assert( 0 );
-            res = _( "This building has been disabled." );
-        }
-        break;
+    case BuildingStatus::BUILD_DISABLE:
+        return _( "This building has been disabled." );
 
-    case LACK_RESOURCES:
-        res = _( "Cannot afford the %{name}." );
+    case BuildingStatus::LACK_RESOURCES: {
+        std::string res = _( "Cannot afford the %{name}." );
         StringReplace( res, "%{name}", GetName() );
-        break;
+        return res;
+    }
 
-    case ALREADY_BUILT:
-        res = _( "The %{name} is already built." );
+    case BuildingStatus::ALREADY_BUILT: {
+        std::string res = _( "The %{name} is already built." );
         StringReplace( res, "%{name}", GetName() );
-        break;
+        return res;
+    }
 
-    case REQUIRES_BUILD:
-        res = _( "Cannot build the %{name}." );
+    case BuildingStatus::REQUIRES_BUILD: {
+        std::string res = _( "Cannot build the %{name}." );
         StringReplace( res, "%{name}", GetName() );
-        break;
+        return res;
+    }
 
-    case ALLOW_BUILD:
-        res = _( "Build %{name}." );
+    case BuildingStatus::ALLOW_BUILD: {
+        std::string res = _( "Build %{name}." );
         StringReplace( res, "%{name}", GetName() );
-        break;
+        return res;
+    }
 
     default:
         break;
     }
 
-    return res;
+    return {};
 }
 
 void BuildingInfo::SetStatusMessage( StatusBar & bar ) const
 {
-    if ( building == BUILD_NOTHING ) {
+    if ( _buildingType == BUILD_NOTHING ) {
         return;
     }
 
-    switch ( bcond ) {
-    case NOT_TODAY:
-    case ALREADY_BUILT:
-    case NEED_CASTLE:
-    case BUILD_DISABLE:
-    case LACK_RESOURCES:
-    case REQUIRES_BUILD:
-    case ALLOW_BUILD:
+    switch ( _status ) {
+    case BuildingStatus::NOT_TODAY:
+    case BuildingStatus::ALREADY_BUILT:
+    case BuildingStatus::NEED_CASTLE:
+    case BuildingStatus::BUILD_DISABLE:
+    case BuildingStatus::SHIPYARD_NOT_ALLOWED:
+    case BuildingStatus::LACK_RESOURCES:
+    case BuildingStatus::REQUIRES_BUILD:
+    case BuildingStatus::ALLOW_BUILD:
         bar.ShowMessage( GetConditionDescription() );
         break;
 
@@ -712,9 +718,9 @@ bool DwellingsBar::ActionBarLeftMouseSingleClick( DwellingItem & dwl )
         castle.RecruitMonster( Dialog::RecruitMonster( { castle.GetRace(), dwType }, castle.getMonstersInDwelling( dwType ), true, -60 ) );
     }
     else if ( !castle.isBuild( BUILD_CASTLE ) )
-        fheroes2::showStandardTextMessage( "", GetBuildConditionDescription( NEED_CASTLE ), Dialog::OK );
+        fheroes2::showStandardTextMessage( "", GetBuildConditionDescription( BuildingStatus::NEED_CASTLE ), Dialog::OK );
     else {
-        const BuildingInfo dwelling( castle, static_cast<building_t>( dwType ) );
+        const BuildingInfo dwelling( castle, static_cast<BuildingType>( dwType ) );
 
         if ( dwelling.DialogBuyBuilding( true ) ) {
             AudioManager::PlaySound( M82::BUILDTWN );
